using System;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Drawing;

class WindowsLibrary : IDisposable
{
	private IntPtr mLibraryHandle = IntPtr.Zero;

	[DllImport("user32.dll", SetLastError=true)]
	public static extern IntPtr LoadIcon(IntPtr hInstance, int lpIconId);

	[DllImport("kernel32.dll", SetLastError=true)]
	static extern IntPtr LoadLibrary(string lpFileName);

	[DllImport("kernel32.dll", SetLastError=true)]
	public static extern int FreeLibrary(IntPtr hModule);

	public WindowsLibrary(string aFileName)
	{
		mLibraryHandle = LoadLibrary(aFileName);
		if (mLibraryHandle != IntPtr.Zero)
		{
			Console.WriteLine("Library {0} loaded.", aFileName);
		} 
		else 
		{
			throw new ArgumentException(
				String.Format("Can not load module {0}", aFileName));
		}
	}

	~WindowsLibrary()
	{
		Dispose(false);
	}

	public void Dispose()
	{
		Dispose(true);
		GC.SuppressFinalize(this);
	}

	private void Dispose(bool aDisposing)
	{
		if (mLibraryHandle != IntPtr.Zero)
		{
			if (aDisposing)
			{
				// Free any managed resources here
			}

			FreeLibrary(mLibraryHandle);
			mLibraryHandle = IntPtr.Zero;
			Console.WriteLine("Library unloaded.");
		}
	}

	public Icon GetIcon(int aIconId)
	{
		IntPtr hIcon = LoadIcon(mLibraryHandle, aIconId);
		if (hIcon == IntPtr.Zero) 
		{
			throw new ArgumentException(String.Format(
				"Can not load icon with id={0}", aIconId));
		}
		Icon icon = Icon.FromHandle(hIcon);
		return icon;
	}
	}

class IDisposableDemo
{
	static void Main()
	{
		WindowsLibrary windowsExplorer = 
			new WindowsLibrary("explorer.exe");

		using (windowsExplorer) 
		{
			const int FIRST_EXPLORER_ICON = 101;
			Icon explorerIcon = 
				windowsExplorer.GetIcon(FIRST_EXPLORER_ICON);

			Form form = new Form();
			form.Text = "IDisposable Demo";
			form.Icon = explorerIcon;
			form.ShowDialog();
		}
		
		Console.WriteLine("End of Main() method.");
	}
}
